<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

class MRWP_History {

    /**
     * Ajax action - Display page content
     */
    public static function display() {
        check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );
        $table = new MRWP_History_List_Table();
        $table->prepare_items();
        $table->display();
        die();
    }

    /**
     * Ajax action - Refresh table content only
     */
    public static function display_refresh() {
        check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );
        $table = new MRWP_History_List_Table();
        $table->prepare_items();
        $table->display_refresh();
    }

    /**
     * Ajax action - Export CSV
     */
    public static function export_csv() {
        check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );
        $table = new MRWP_History_List_Table();
        $all_records = true;
        $table->prepare_items( $all_records );
        $table->export_csv();
    }

    /**
     * Ajax action - Update shipping status
     */
    public static function update_status() {
        check_ajax_referer( 'mrwp-list-nonce', 'mrwp_list_nonce' );

        $expedition_num = ! empty( $_REQUEST['shipment_number'] ) ? $_REQUEST['shipment_number'] : 0;

        if ( 0 != $expedition_num ) {

            // Find order id for this shipping number
            $args = array(
                'limit'      => 1,
                'meta_query' => array( [
                    'key'   => 'ExpeditionNum',
                    'value' => $expedition_num,
                ] ),
                // 'return'      => 'ids',
            );
            $orders = wc_get_orders( $args );

            if ( 1 == count( $orders ) ) {
                $order = $orders[0];

                $response = new stdClass();

                /*
                 * We don’t want to overload the MR servers
                 *
                 * Check that the last update was more than 1 hour ago
                 */
                $timestamp = $order->get_meta( 'ExpeditionStatusDate' );
                if ( time() - $timestamp < 60 * 60 ) {
                    $response->code   = false;
                    $response->status = 'too-fast';

                    die( json_encode( $response ) );
                }

                // @todo All of the following should be moved to an API class
                $params_compte = class_MRWP_order::get_params_compte();

                $client = new SoapClient( class_MRWP_main::MONDIALRELAY_WEBSERVICE );

                $shop_location = wc_get_base_location();
                $langue        = class_MRWP_helpers::get_langue( $shop_location['country'] );
                $params = array(
                    'Enseigne'       => $params_compte['mondialrelay_code_client'],
                    'Expedition'    => $expedition_num,
                    'Langue'         => $langue,
                );

                // Generate the security key
                $code = implode("", $params);
                $code .= $params_compte['mondialrelay_cle_privee'];
                $params["Security"] = strtoupper(md5($code));

                try {
                    $tracking = $client->WSI2_TracingColisDetaille( $params )->WSI2_TracingColisDetailleResult;
                    $response->code = $tracking->STAT;
                    $response->text = class_MRWP_helpers::statut( $tracking->STAT );

                    // Update the database
                    $previous_status = $order->get_meta( 'ExpeditionStatus' );

                    if ($tracking->STAT == 80 || $tracking->STAT == 81 || $tracking->STAT == 82){
                        $order->update_meta_data( 'ExpeditionStatus', $tracking->STAT );
                        $order->update_meta_data( 'ExpeditionStatusDate', time() );
                        $order->save();
                    // Incorrect tracking result
                    } elseif ($tracking->STAT == 83){
                        $order->update_meta_data( 'ExpeditionStatus', $tracking->STAT );
                        $order->update_meta_data( 'ExpeditionStatusDate', time() );
                        $order->save();
                    // Error
                    } else {
                    }

                    if ( 82 == $tracking->STAT ) {
                        $order_delivered_status = get_option( 'mondialrelay_state_order_delivered', 0 );
                        if ( '' != $order_delivered_status ) {
                            $order->update_status( $order_delivered_status );
                        }
                    }
                    if ( $previous_status == $tracking->STAT ) {
                        $response->status = 'unchanged';
                    } else {
                        $response->status = 'updated';
                    }

                } catch( Exception $e ){
                    $response->code = 789;
                    $response->status = 'error';
                    $response->text = class_MRWP_helpers::statut( 789 );
                }

                die( json_encode( $response ) );
            }
        }

        // No shipping number or order not found
        $response = new stdClass();
        $response->code   = false;
        $response->text   = 'Function called without a shipping number';
        $response->status = 'error';

        die( json_encode( $response ) );
    }

    /**
     * Filter - Add meta parameters to query
     *
     * WooCommerce ignores the meta_query arg by default, so we add it ourselves
     */
    public static function get_orders_meta( $query, $query_vars ) {
        if ( isset( $query_vars['meta_query'] ) ) {
            $query['meta_query'] = $query_vars['meta_query'];
        }

        return $query;
    }

}
